"""Example for visualizing a robot."""

import time

from asyncua.sync import Client

from voraus_3d_visu import Visu

model = "http://voraus-core/robots/VORAUS_INDUSTRIAL_ROBOT/VORAUS_INDUSTRIAL_ROBOT_002.glb"


if __name__ == "__main__":
    visu = Visu("http://voraus-3d-visu/", clear_all=True)
    client = Client("opc.tcp://voraus-core:48401/")

    with visu.connection(), client:
        robot = visu.add_model(model_url=model, position=[0, 0, 0])
        joint_positions_node = client.get_node("ns=1;i=100111")

        while True:
            (joint_positions,) = client.read_values([joint_positions_node])

            visu.update(
                robot.child("CS0").rotation.z(joint_positions[0]),
                robot.child("CS1").rotation.z(joint_positions[1]),
                robot.child("CS2").rotation.z(joint_positions[2]),
                robot.child("CS3").rotation.z(joint_positions[3]),
                robot.child("CS4").rotation.z(joint_positions[4]),
                robot.child("CS5").rotation.z(joint_positions[5]),
            )

            time.sleep(0.01)



"""Example for moving a robot."""

from voraus_robot_arm import JointPose, VorausIndustrialRobotArm, x

robot = VorausIndustrialRobotArm()
home = JointPose(0, -1.57, 1.57, -1.57, -1.57, 0)

if __name__ == "__main__":
    robot = VorausIndustrialRobotArm()
    with robot.connect("voraus-core", port=48401):
        robot.enable()
        robot.move_ptp(home).result()

        input("Press enter to move the robot.")
        robot.move_linear_relative(x(0.2)).result()



"""Animation utilities for the transformation example."""

import time
from typing import Self


class Animation:
    """An animation iterator."""

    def __init__(self, start: float, stop: float, duration: float, frequency: float = 25) -> None:
        """Initializes the animation iterator.

        Args:
            start: The start value for the animation.
            stop: The end value for the animation.
            duration: The animation duration in seconds.
            frequency: The frequency for animation steps. Defaults to 25 Hz.
        """
        self.frequency = frequency
        self.steps = round(frequency * duration)
        self.values = iter(start + (i * ((stop - start) / self.steps)) for i in range(self.steps + 1))

    def __iter__(self) -> Self:
        """Returns self as an iterator."""
        return self

    def __next__(self) -> float:
        """Returns the next iteration step of the animation and sleeps."""
        time.sleep(1 / self.frequency)
        return next(self.values)



"""Example for object and children transformation.

All children A, A1 and B are in the same model with the coordinate system 'obj'.
All transformations applied to 'obj' do also influence A, A1 and B.
A1 is linked to A such that A is the parent of A1.
"""

from math import radians
from pathlib import Path

from animation import Animation

from voraus_3d_visu import Visu

visu = Visu("http://voraus-3d-visu/", clear_all=True)
model = Path(__file__).parent / "transforms.glb"


if __name__ == "__main__":
    with visu.connection():
        obj = visu.add_model(model)
        # Move the object
        input("Press enter to move the object with all children.")
        for step in Animation(0, -1, duration=1):
            visu.update(obj.position.y(step))
        # Rotate the object
        input("Press enter to rotate the object with all children")
        for step in Animation(0, radians(25), duration=1):
            visu.update(obj.rotation.z(step))
        # Reload object with initial transforms
        input("Press enter to reload model with initial transforms.")
        visu.client.clear()
        obj = visu.add_model(
            model,
            position=[-0.5, -0.5, 0],
            rotation=[0, 0, radians(26)],
        )
        # Rotate B
        input("Press enter to rotate B in obj CS.")
        for step in Animation(0, radians(90), duration=1):
            visu.update(obj.child("B").rotation.x(step))
        # Move B
        input("Press enter to move B in obj CS.")
        for step in Animation(0, 1.0, duration=1):
            visu.update(obj.child("B").position.y(step))
        # Rotate A
        input("Press enter to rotate A in obj CS.")
        for step in Animation(0, radians(-20), duration=1):
            visu.update(obj.child("A").rotation.y(step))
        # Move A1
        input("Press enter to move A1 in A CS.")
        for step in Animation(1.0, 0, duration=1):
            visu.update(obj.child("A1").position.x(step))



"""Example for visualizing a box."""

from pathlib import Path

from voraus_3d_visu import Visu

visu = Visu("http://voraus-3d-visu/", clear_all=True)
model = Path(__file__).parent / "box.glb"


if __name__ == "__main__":
    with visu.connection():
        box = visu.add_model(model, position=[0, 0, 0])

        # Move and rotate the box
        input("Press enter to move and rotate the box.")
        visu.update(
            box.position.x(0.3),
            box.rotation.z(1.57),
        )

        # Make the box invisible
        input("Press enter to make the box invisible.")
        visu.update(box.visible(False))

        # Make the box visible
        input("Press enter to make the box visible.")
        visu.update(box.visible(True))

        # Add an axes helper object
        input("Press enter to add axes helpers and parent them to the box.")
        axes_object = visu.add_axes(position=[0.3, 0, 0], rotation=[0, 0, 1.57], scale=[0.5, 0.5, 0.5])
        visu.update(axes_object.parent(box))

        # Add a second invisible box
        input("Press enter to add another invisible box.")
        box2 = visu.add_model(model, position=[1, 0, 0], visible=False)

        # Make the second box visible
        input("Press enter to make the second box visible.")
        visu.update(box2.visible(True))

        input("Press enter to add a text panel")
        text_panel = visu.add_text_panel(text="Position [-1, 0, 0]", position=[-1, 0, 0])

        input("Press enter to reference the text panel to the box")
        visu.update(text_panel.parent(box), text_panel.position.xyz(0, 0, 0), text_panel.text("Box"))

        input("Press enter to hide the text panel")
        visu.update(text_panel.visible(False))

        # Add hash instructions
        visu.add_hash_instruction("invisible", box.visible(False), box.position.x(-5), box.scale.z(0.2))
        visu.add_hash_instruction("visible", box.visible(True))

        # Exit the program
        input("Press enter to exit the program.")



"""Example for using a reverse proxy."""

from pathlib import Path

from voraus_3d_visu import Visu

visu = Visu("http://voraus-3d-visu/", clear_all=True)
model = Path(__file__).parent / "box.glb"


if __name__ == "__main__":
    with visu.connection():
        box = visu.add_model(model, position=[0, 0, 0])
        # Move and rotate the box
        input("Press enter to move and rotate the box.")
        visu.update(
            box.position.x(0.3),
            box.rotation.z(1.57),
        )
        # Exit the program
        input("Press enter to exit the program.")
